;============================================================
; DMA Transfer setup routines
;
; Author   : Bjorge Dijkstra (bjorge@gmx.net)
; Date     : 03-03-2000
; version  : 1.1
;
; revisions
; ---------
; 23-03-2000	- fixed 16bit dma bug
;============================================================

;============================================================
; Typical use :
;
; - Setup DMA controller with a call to DMAInitTransfer
; - Setup other device for DMA transfer
;
; DMA transfer usually starts immediately after the other
; device has been setup.
;
;============================================================
;
; To setup a transfer call DMAInitTransfer with:
;
; AL  = transfer mode
; AH  = DMA channel
; CX  = Size (max 64kb)
; EDI = Address of block
;
;============================================================

;============================================================
; constants
;============================================================

 dmaModeREQ		equ  000h
 dmaModeSINGLE		equ  040h
 dmaModeBLOCK		equ  080h
 dmaModeCASCADE    	equ  0C0h
 dmaAddrDec           	equ  020h
 dmaAddrInc           	equ  000h
 dmaEnableAutoInit    	equ  010h
 dmaDisableAutoInit   	equ  000h
 dmaVrfyTransfer      	equ  000h
 dmaWrTransfer        	equ  004h
 dmaRdTransfer        	equ  008h

 dmaSetReqBit         	equ  04h
 dmaClrReqBit         	equ  00h
 dmaSetMskBit         	equ  04h
 dmaClrMskBit         	equ  00h


;============================================================
; DMA Port tables
;============================================================

 DMA_Address           db   000h,002h,004h,006h,0C0h,0C4h,0C8h,0CCh
 DMA_Length            db   001h,003h,005h,007h,0C2h,0C6h,0CAh,0CEh
 DMA_RdStatus          db   008h,008h,008h,008h,0D0h,0D0h,0D0h,0D0h
 DMA_WrStatus          db   008h,008h,008h,008h,0D0h,0D0h,0D0h,0D0h
 DMA_WrReq             db   009h,009h,009h,009h,0D2h,0D2h,0D2h,0D2h
 DMA_WrSnglMsk         db   00Ah,00Ah,00Ah,00Ah,0D4h,0D4h,0D4h,0D4h
 DMA_WrMode            db   00Bh,00Bh,00Bh,00Bh,0D6h,0D6h,0D6h,0D6h
 DMA_Clr               db   00Ch,00Ch,00Ch,00Ch,0D8h,0D8h,0D8h,0D8h
 DMA_RdTemp            db   00Dh,00Dh,00Dh,00Dh,0DAh,0DAh,0DAh,0DAh
 DMA_MasterClr         db   00Dh,00Dh,00Dh,00Dh,0DAh,0DAh,0DAh,0DAh
 DMA_ClrMsk            db   00Eh,00Eh,00Eh,00Eh,0DCh,0DCh,0DCh,0DCh
 DMA_WrAllMsk          db   00Fh,00Fh,00Fh,00Fh,0DEh,0DEh,0DEh,0DEh
 DMA_LowerPage         db   087h,083h,081h,082h,000h,08Bh,089h,08Ah
 DMA_HigherPage        dw   487h,483h,481h,482h,000h,48Bh,489h,48Ah

 
printEDX:
	push	eax
	push	ebx
	push	edi

	push	eax	
	mov	ebx,edx
	mov	ax,0405h
	int	20h
	mov	al,04h
	mov	edi,CRLF
	int	20h
	pop	eax

	mov	ebx,eax
	mov	ax,0405h
	int	20h
	mov	al,04h
	mov	edi,CRLF
	int	20h
	pop	edi
	pop	ebx
	pop	eax
	ret

;============================================================
; DMASetMode
; ----------
; IN:
;    AL = Mode
;    AH = Channel
;============================================================
DMASetMode:
	push esi			; save used registers
	push edx
	push eax
	
        xor  dx,dx			; clear dx
        mov  si,ax			; load channel to si
        shr  si,8
        and  si,07h			; esi = index into table
        mov  dl,[DMA_WrMode + si]	; get port number
        and  ah,03h
        add  al,ah
        out  dx,al			; output mode to channel
        
        pop  eax
        pop  edx
        pop  esi
        ret				; return

;============================================================
; DMAClearBytePtr
; ----------------
; IN:
;    AH = Channel
;============================================================
DMAClearBytePtr:
	push esi			; save used registers
	push edx
	push eax

        xor  dx,dx			; clear dx
        mov  si,ax			; load channel to si
        shr  si,8
        and  si,07h			; si = index into table
        mov  dl,[DMA_Clr + si]		; get port number
        mov  al,0
        out  dx,al			; clear
        
	pop  eax
        pop  edx
        pop  esi
        ret				; return	


;============================================================
; DMASetStartAddress
; ------------------
; IN:
;    AH   = Channel
;    EDI  = Address of block  
;============================================================
DMASetStartAddress:
	push esi			; save used registers
	push edx
	push ebx
	push eax

	test ah,0FCh
	jz   DMASSA_8bit		; 8 bit DMA channel?
	mov  ebx,edi			; with 16bit dma channels
	and  ebx,10000h			; we need to do some
	shr  ebx,1			; other calc. on address
	shr  di,1
	and  di,7fffh
        or   di,bx

DMASSA_8bit:
        mov  si,ax			; load channel to si
        shr  si,8
        and  si,07h			; si = index into table

        mov  eax,edi			; eax is block address
        xor  dx,dx			; clear dx
        mov  dl,[DMA_Address + si]	; get DMA address port number
        out  dx,al			; send Lower  byte
        mov  al,ah			
        out  dx,al			; send Higher byte
	mov  dl,[DMA_LowerPage + si]
	shr  eax,16			; shift blocks page address to ax
        out  dx,al			; output lower page byte
	shl  si,1
	mov  dx,[DMA_HigherPage + si]	;
        mov  al,ah
	out  dx,al			; output higher page byte

        pop  eax
        pop  ebx
	pop  edx
	pop  esi
	ret


;============================================================
; DMASetBlockSize
; ---------------
; IN:
;    AH   = Channel
;    CX   = block size
;============================================================
DMASetBlockSize:
	push esi			; save used registers
	push ecx
	push eax
	push edx

	mov  si,ax			; load channel to si
	shr  si,8			;
	and  si,07h			; si = index into table
	xor  dx,dx
	mov  dl,[DMA_Length + si]	; get port from table
	mov  al,cl			; output low byte of size
        out  dx,al
	mov  al,ch			; output high byte of size
	out  dx,al
	
	pop  edx
	pop  eax			; restore registers
	pop  ecx
	pop  esi
	ret

;============================================================
; DMADisableChannel
; ----------------
; IN:
;    AH   = Channel
;============================================================
DMADisableChannel:
	push esi			; save registers
	push eax
	push edx

	shr  ax,8			; shift channel to al
	mov  si,ax			; 
	and  si,07h			; si = inde into table
	xor  dx,dx			
	mov  dl,[DMA_WrSnglMsk + si]	; get port from table

	or   al,dmaSetMskBit
	out  dx,al			; disable channel
	
	pop  edx			; restore registers
	pop  eax
	pop  esi
	ret	

;============================================================
; DMAEnableChannel
; ----------------
; IN:
;    AH   = Channel
;============================================================
DMAEnableChannel:
	push esi			; save registers
	push eax
	push edx

	shr  ax,8			; shift channel to al
	mov  si,ax
	and  si,07h			; si = index into table
	xor  dx,dx
	mov  dl,[DMA_WrSnglMsk + si]	; get port from table
	and  al,03h
	out  dx,al			; enable channel

	pop  edx			; restore registers
	pop  eax
	pop  esi
	ret	

;============================================================
; DMAInitTransfer
; ----------------
; IN:
;    AL   = Mode
;    AH   = Channel
;    CX   = Size
;    EDI  = Address
;============================================================
DMAInitTransfer:
	call DMADisableChannel		; disable dma channel
	call DMAClearBytePtr		; clear byte pointer
	call DMASetMode			; set transfer mode
	call DMASetStartAddress		; set address of block to transfer
	call DMASetBlockSize		; set block size
	call DMAEnableChannel		; enable dma channel
	
	
	ret

