;=============================================================
; SoundBlaster DSP routines (not complete)
;
; Author   : Bjorge Dijkstra (bjorge@gmx.net)
; Date     : 03-03-2000
;
;=============================================================


;=============================================================
;
;  DSP Routines:
;
;  DSPReset			- Reset DSP
;  DSPDetectBase		- Find DSP Baseport
;  DSPWrite			- Write byte to DSP
;  DSPRead			- Read byte from DSP
;  DSPGetVersion		- Get DSP version
;  DSPSpeakerOn			- Turns speakers on
;  DSPSpeakerOff		- Turns speakers off
;  DSPGetSpeaker		- Get speaker status
;  DSPSetSampleRate		- Sets sample rate
;=============================================================



;=============================================================
; DSPReset
; --------
; IN :
;       DX - DSP base port
; OUT:
;       carry set    = failed
;             clear  = success              
;=============================================================
DSPReset:
        push ax                  ; save used registers
        push cx
        push dx

        add  dl,6                ; select DSP reset port (DX = base+06h)
        mov  al,1                ;
        out  dx,al               ; send AL (=1) to reset port
        mov  eax,0FFFh
ResetDSP_delay:                  ; simple delay loop
        dec  al                  ;
        jnz  ResetDSP_delay      ; decrease AL until zero
        out  dx,al               ; send AL (=0) to reset port
        mov  cx, 0FFFh
        add  dl,8                ; select DSP data ready port (dx = base+0eh)
ResetDSP_Status: 
        in   al,dx               ; 
        or   al,al               ; data ready to be read?
        jns  ResetDSP_Retry      ; if not, try again
        sub  dl,4                ; select DSP data port (dx = base + 0ah) 
        in   al,dx               ; read DSP data
        cmp  al,0aah             ; is al expected value?
        je   ResetDSP_OK         ; yes, then reset was successfull
        add  dl,4
ResetDSP_Retry:
        loop ResetDSP_Status     ; try to read status value again
        stc                      ; all attempts failed,  set carry
        jmp  ResetDSP_exit
ResetDSP_Ok:
        clc                      ; success, clear carry
ResetDSP_exit:
        pop  dx
        pop  cx                  ; restore saved registers
        pop  ax 
        ret                      ; and return to caller


;=============================================================
; DSPDetectBase
; -------------
; OUT:
;       carry   =  clear, success
;                  set,   failed
;       DX      =  if success, DSP base port
;=============================================================  
DSPDetectBase:
        mov  dx,0210h                ; First valid DSP port
DetectDSPBase_TryPort:        
        call DSPReset                ; try to reset DSP
        jnc  DetectDSPBase_OK
        add  dx,10h                  ; try next port
        cmp  dx,0290h                ; tried all ports?
        je   DetectDSPBase_Fail      
        jmp  DetectDSPBase_TryPort 
DetectDSPBase_OK:
        clc                          ; clear carry
        jmp  DetectDSPBase_End   
DetectDSPBase_Fail:
        stc                          ; set carry   
DetectDSPBase_End:
        ret                          ; return to caller   


;=============================================================
; DSPWrite
; --------
; IN :
;       DX = DSP base port
;       AL = Data byte
;=============================================================
DSPWrite:
        push edx         
        push eax
        add  dx,0ch              ; select write data port
WriteDSP_Busy:
        in   al,dx               ; get write status
        and  al,80h              ; ready to write ?
        jnz  WriteDSP_Busy       ; no check status again
        pop  eax 
        out  dx,al               ; write data byte
        pop  edx
        ret                      ; return to caller


;=============================================================
; DSPRead
; --------
; IN :
;       DX - DSP base port
; OUT:
;       AL - Data byte
;=============================================================
DSPRead:
        push edx
        add  dx,0eh              ; select data ready status port
ReadDSP_Busy:
        in   al,dx               ; get status
        and  al,80h
        jz   ReadDSP_Busy
        sub  dx,4                ; select data read port
        in   al,dx               ; read byte
        pop  edx
        ret                      ; return to caller


;=============================================================
; DSPGetVersion
; -------------
; IN :
;       DX - DSP base port
; OUT:
;       AH - Version Major
;       AL - Version Minor
;=============================================================
DSPGetVersion:	
	mov  al, 0E1h		; command E1h = Get DSP Version
        call DSPWrite           ; send command
        call DSPRead            ; read Major
        shl  ax,8 
        call DSPRead		; read Minor
        ret


;=============================================================
; DSPSpeakerOn
; -------------
; IN :
;       DX - DSP base port
;=============================================================
DSPSpeakerOn:
	mov  al,0D1h
	call DSPWrite
        ret


;=============================================================
; DSPSpeakerOff
; -------------
; IN :
;       DX - DSP base port
;=============================================================
DSPSpeakerOff:
	mov  al,0D3h
	call DSPWrite
        ret

;=============================================================
; DSPGetSpeaker
; -------------
; IN :
;       DX - DSP base port
; OUT
;       AL - 01h = off
;            FFh = on
;=============================================================
DSPGetSpeaker:
	mov  al,0D8h
	call DSPWrite
        call DSPRead
        ret

;=============================================================
; DSPSetSampleRate
; ----------------
; IN :
;       DX - DSP base port
;       EBX - Sample rate in Hz
;
; TC = 256 - (1000000 div Rate)
;=============================================================
DSPSetSampleRate:
	push eax		; save used registers
	push ebx
	push edx
	
        mov  al,40h		; Set Sample Rate DSP command
	call DSPWrite		; 	

	push edx		; save DSP Base Port
	
        xor  edx,edx		; edx = 0
        mov  eax,1000000	; eax = 1.000.000
	div  ebx		; calc. eax = (edx:eax) div ebx
        neg  al			; al = 256 - al
        
	pop  edx		; restore DSP Base Port
	call DSPWrite		; Write TC (=AL) to DSP
	
	pop  edx		; restore registers
	pop  ebx
	pop  eax
	ret

